unit GWDragDemoAppInterface;

{---------------------------------------------------------------
Purpose: Implements App-to-DragSource interface for GWDragDemo app.

Author:  Graham Wideman
         graham@wideman-one.com
         www.wideman-one.com

TGWDragDemoAppInterface
-------------
Demos how to subclass TGWDragAppInterface and implement the
app-specific portion of an OLE-drag-sourcing application.

NOTE: We are overriding most of the GWDragAppInterface ONLY in order
to show all the input and output arguments diagnostically.
Normally you don't need to override all these. 

Revision History
----------------
98-07-21 GW original GWDragDemoAppInterface

To-do list
----------

---------------------------------------------------------------}

interface
uses
  Windows, ActiveX, Classes, forms,
  GWDragSource;


Type
  //------------------------------------
  TGWDragDemoAppInterface = class(TGWDragAppInterface)
  //------------------------------------
  public
    function DoDragDrop(dwOKEffects: Longint; var dwEffect: Longint): HResult; override;

  protected
    //--------------------------------------------
    // implementations for the delegated functions
    // mostly to add monitoring of arguments
    //--------------------------------------------
    //TGWDropSource
    function QueryContinueDrag(fEscapePressed: BOOL; grfKeyState: Longint): HResult; override;
    function GiveFeedback(dwEffect: Longint): HResult; override;

    //TGWDragDataObject
    function QueryGetData(const formatetc: TFormatEtc): HResult; override;
    function GetData     (const formatetc: TFormatEtc; var medium: TStgMedium): HResult; override;

    //TGWDragEnumFormatEtc
    function FormatCount: integer; override;
    procedure FormatEtc(N: integer; var formatetc: TFormatEtc); override;

    //-------------------------------------------
    // Utility functions
    //-------------------------------------------
    function EffectString(dwEffect: longint): string;
    function KeyStateString(grfKeyState: longint): string;
  end;

//--------------------------------
implementation
//--------------------------------
uses sysutils, stdctrls, graphics, GWDragDemo_Main;  // so we can fiddle with form directly

//---------------------------------------
function TGWDragDemoAppInterface.DoDragDrop(dwOKEffects: Longint; var dwEffect: Longint): HResult;
//---------------------------------------
Begin
  (FOwnerForm as Tfm_Main).lb_OKEffects.Caption := EffectString(dwOKEffects);
  (FOwnerForm as Tfm_Main).lb_DDDEffect.Caption  := '---';
  (FOwnerForm as Tfm_Main).lb_DDD_Result.Caption := '---';

  Result := inherited DoDragDrop(dwOKEffects, dwEffect);
  (FOwnerForm as Tfm_Main).lb_DDDEffect.Caption  := EffectString(dwEffect);
  (FOwnerForm as Tfm_Main).lb_DDD_Result.Caption := IntToHex(Result,8);

end;

//TGWDropSource
//---------------------------------------
function TGWDragDemoAppInterface.QueryContinueDrag(fEscapePressed: BOOL; grfKeyState: Longint): HResult;
//---------------------------------------
Begin
  (FOwnerForm as Tfm_Main).lb_KeyState.Caption := KeyStateString(grfKeyState);
  Result := inherited QueryContinueDrag(fEscapePressed, grfKeyState);

  (FOwnerForm as Tfm_Main).lb_QCD_Result.Caption := IntToHex(Result,8);
end;

//---------------------------------------
function TGWDragDemoAppInterface.GiveFeedback(dwEffect: Longint): HResult;
//---------------------------------------
Begin
  (FOwnerForm as Tfm_Main).lb_Effect.Caption := EffectString(dwEffect);
  Result := inherited GiveFeedback(dwEffect);
  (FOwnerForm as Tfm_Main).lb_GF_Result.Caption := IntToHex(Result,8);
end;

//---------------------------------------
function TGWDragDemoAppInterface.QueryGetData(const formatetc: TFormatEtc): HResult;
//---------------------------------------
Begin
  Result := DV_E_FORMATETC;
  if (formatetc.dwAspect = DVASPECT_CONTENT) and
     ((formatetc.cfFormat = CF_TEXT) or (formatetc.cfFormat = CF_UNICODETEXT)) and
     (formatetc.tymed = TYMED_HGLOBAL) then Result := S_OK;
end;

//---------------------------------------
function TGWDragDemoAppInterface.GetData(const formatetc: TFormatEtc; var medium: TStgMedium): HResult;
//---------------------------------------
Var
  HGlobalData: HGlobal;
  PGlobalData: PChar;
  DemoString: string;
  DemoPChar:  PChar;
  DemoStringLen: integer;
  AllocLen: integer;

Label the_end;
Begin
  Result := DV_E_FORMATETC;
  If QueryGetData(formatetc) <> S_OK then goto the_end;

  // most medium fields are already initialized by this point
  medium.tymed := TYMED_HGLOBAL;

  DemoString     := (FOwnerForm as Tfm_Main).tx_DragSource.text;
  DemoStringLen  := Length(DemoString);

  Case formatetc.cfFormat of
    CF_TEXT:        AllocLen := DemoStringLen + 1;
    CF_UNICODETEXT: AllocLen := DemoStringLen * 2 + 2;
  end;

  HGlobalData := GlobalAlloc((GMEM_SHARE or GMEM_MOVEABLE or GMEM_ZEROINIT), AllocLen);
  if (HGlobalData <> 0) then
  begin
    PGlobalData := GlobalLock (HGlobalData); // lock while we are using it
    Case formatetc.cfFormat of
      CF_TEXT:        StrCopy(PGlobalData, PChar(DemoString));
      CF_UNICODETEXT: StringToWideChar(DemoString, PWideChar(PGlobalData), DemoStringLen+1);
    end;
    GlobalUnlock (HGlobalData);
    medium.hGlobal := HGlobalData;
    Result := S_OK;
  end;
the_end:
end;


//---------------------------------------
function TGWDragDemoAppInterface.FormatCount: integer;
//---------------------------------------
Begin // dumb default implementation
  Result := 2;
end;

//---------------------------------------
procedure TGWDragDemoAppInterface.FormatEtc(N: integer; var formatetc: TFormatEtc);
//---------------------------------------
Begin
  Case N of
    1: Begin
      formatetc.cfFormat := CF_UNICODETEXT;
      formatetc.ptd      := nil;
      formatetc.dwAspect := DVASPECT_CONTENT;
      formatetc.lindex   := -1;
      formatetc.tymed    := TYMED_HGLOBAL;
    end;
    else Begin
      formatetc.cfFormat := CF_TEXT;
      formatetc.ptd      := nil;
      formatetc.dwAspect := DVASPECT_CONTENT;
      formatetc.lindex   := -1;
      formatetc.tymed    := TYMED_HGLOBAL;
    end;
  end;
end;

//--------------------------------------------------
function TGWDragDemoAppInterface.KeyStateString(grfKeyState: Longint): string;
//--------------------------------------------------
Var S: string;
  procedure Add(SAdd: string);
  Begin
    If S <> '' then S := S + '+';
    S := S + SAdd;
  end;
Const
  MK_ALT = $020;  // this is not defined with the others for some reason

Begin
  S := '';
  If (grfKeyState and MK_SHIFT  ) <> 0 then Add('Sh');
  If (grfKeyState and MK_CONTROL) <> 0 then Add('Ctl');
  If (grfKeyState and MK_ALT    ) <> 0 then Add('Alt');
  If (grfKeyState and MK_LBUTTON) <> 0 then Add('ML');
  If (grfKeyState and MK_MBUTTON) <> 0 then Add('MM');
  If (grfKeyState and MK_RBUTTON) <> 0 then Add('MR');
  Result := IntToHex(grfKeyState, 8)+': '+S;
end;

//--------------------------------------------------
function TGWDragDemoAppInterface.EffectString(dwEffect: Longint): string;
//--------------------------------------------------
Var S: string;
  procedure Add(SAdd: string);
  Begin
    If S <> '' then S := S + '+';
    S := S + SAdd;
  end;
Const
  MK_ALT = $020;  // this is not defined with the others for some reason

Begin
  S := '';
  If (dwEffect and DROPEFFECT_COPY  ) <> 0 then Add('Copy');
  If (dwEffect and DROPEFFECT_MOVE  ) <> 0 then Add('Move');
  If (dwEffect and DROPEFFECT_LINK  ) <> 0 then Add('Link');
  If (dwEffect and DROPEFFECT_SCROLL) <> 0 then Add('Scroll');
  Result := IntToHex(dwEffect, 8)+': '+S;
end;


end.

